#! /usr/bin/wish -f
# -*- tcl-*-

# AVRConfig pre2 0.1
#
# this program should help you configuring you libraries...
# 
# released under GPL by Johann Wilhelm oe6jwf
#

global VarList
global VarDesc
global Var2ModulMapper
global Notices

global AvailableModules

#displayed in ModulSelection Dlg
global ModulToolTip
#printed to config
global ModulDescription 

set AvailableModules ""
set Notices ""


# removes a variable from list
# returns 0 if var not registered
proc RemoveVariable {modul var} {
	global VarList
	global VarDesc
	global Var2ModulMapper
	if ([lsearch [array names Var2ModulMapper] $var]==-1) {
		return 0
	}
	set Var2ModulMapper($var) [lreplace $Var2ModulMapper($var) [lsearch $Var2ModulMapper($var) $modul] [lsearch $Var2ModulMapper($var) $modul]]
	if ("$Var2ModulMapper($var)"=="") {
		array unset Var2ModulMapper $var
		array unset VarList $var
		array unset VarDesc $var
	}
	return 1
}


# Gets the Value of a variable
proc GetVariableValue {var} {
	global VarList
	return $VarList($var)
}

# Sets a description for a modul
proc SetModulDescription {modul value} {
	global ModulDescription
	set  ModulDescription($modul) "$value"
}

# Sets a ToolTip for a modul
proc SetModulToolTip {modul value} {
	global ModulToolTip
	set  ModulToolTip($modul) "$value"
}

# Sets a variable description
proc SetVariableDescription {var value} {
	global VarDesc
	set  VarDesc($var) "$value"
}

# Sets a variable value
proc SetVariableValue {var value} {
	global VarList
	set  VarList($var) "$value"
}


# Registers a Variable ...
# returns 0 if a variable has been used more than once else returns 1
proc RegisterVariable {modul var} {
	global VarList
	global Notices
	global Var2ModulMapper
	if {[lsearch [array names VarList] $var]!=-1} {
		set Notices [linsert $Notices end "// notice: $var is used more than one time"]
		set  Var2ModulMapper($var) [linsert $Var2ModulMapper($var) end $modul]		
		return 0
	} else {
		set  VarList($var) ""
		set  Var2ModulMapper($var) ""
		set  Var2ModulMapper($var) [linsert $Var2ModulMapper($var) end $modul]
		return 1
	}
}

# returns 1 if the specified variable has been registered
proc IsVariableRegistered {var} {
	global VarList
	global Notices
	global Var2ModulMapper
	if {[lsearch [array names VarList] $var]!=-1} {
		return 1
	} else {
		return 0
	}
}

# saves everything to config.h
proc Save {} {
	global VarList
	global VarDesc
	global Var2ModulMapper
	
	global Notices
	global AvailableModules
	global ModulDescription 
	
	#sort the modules...
	set AvailableModules [lsort $AvailableModules]

	# output the configured variables

	set configfile [open "config.h" w+]

	puts $configfile "/* This file was produced by AVRConfig 0.1 */\n"
	foreach  notice $Notices {
		puts $configfile $notice
	}
	
	# output variables used in multible modules...
	puts $configfile "\n/* following variables or used in more than one module */"
	foreach  var [lsort [array names Var2ModulMapper]] {
		if ([llength $Var2ModulMapper($var)]>1) {
			puts $configfile "\n// used by $Var2ModulMapper($var)"
			puts $configfile "#define $var $VarList($var)"
		}
	}

	puts $configfile "\n/* Modul Configuration */"

	foreach  modul $AvailableModules {
		global $modul\_selected
		puts $configfile ""
		if ($$modul\_selected==1) {
			puts $configfile "// Modul $modul"
			#print Modul Description if set
			if ([lsearch [array names ModulDescription] $modul]!=-1) {
				puts $configfile "/*\n$ModulDescription($modul)\n*/"
			}
			foreach var [lsort [array names VarList]] {
				if ([array size $Var2ModulMapper($var)]<=1) {
					if ([lsearch $Var2ModulMapper($var) $modul]!=-1) {
						#if any description has been set output it ;)
						if ([lsearch [array names VarDesc] $var]!=-1) {
							puts $configfile "/*\n$VarDesc($var)*/"
						}
						puts $configfile "#define $var $VarList($var)"
					}
				}
			}
		}
	}
	close $configfile
}

# Ballon functions borrowed from http://wiki.tcl.tk/534
proc balloon {w help} {
    bind $w <Any-Enter> "after 1000 [list balloon:show %W [list $help]]"
    bind $w <Any-Leave> "destroy %W.balloon"
  }
  proc balloon:show {w arg} {
    if {[eval winfo containing  [winfo pointerxy .]]!=$w} {return}
    set top $w.balloon
    catch {destroy $top}
    toplevel $top -bd 1 -bg black
    wm overrideredirect $top 1
    if {$::tcl_platform(platform) == "macintosh"} {
     unsupported1 style $top floating sideTitlebar
    }
    pack [message $top.txt -aspect 10000 -bg lightyellow \
            -font fixed -text $arg]
    set wmx [winfo rootx $w]
    set wmy [expr [winfo rooty $w]+[winfo height $w]]
    wm geometry $top \
      [winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
    raise $top
 }
  
#shows the ModulSelect Dialog
proc ShowModulSelect {} {
	global VarList
	global Notices
	global ChoosenModules
	global AvailableModules
	global ModulToolTip

	toplevel .modulselect -width 250 -height 240
	grab .modulselect

	set parent .mainframe
	set child .modulselect

	wm title .modulselect "Modul Selector"
	set y 6
	
	foreach modul $AvailableModules {
		global $modul\_selected
		checkbutton .modulselect.[string tolower $modul] -text "$modul" -variable $modul\_selected
		# if any description has been set.. set the tooltip
		puts [array names ModulToolTip]
		if ([lsearch [array names ModulToolTip] $modul]!=-1) {
			balloon .modulselect.[string tolower $modul] $ModulToolTip($modul)
		}
		place .modulselect.[string tolower $modul] -in .modulselect -x 2 -y $y
		set y [expr {$y+20}]
	}
	
	button .modulselect.ok -text "OK" -command {ModulSelect_OK} -width 10 -default active
	balloon .modulselect.ok "Push me if you're done with this"
	place .modulselect.ok -in .modulselect -x 90 -y [expr {$y+10}]

	.modulselect  configure -height [expr {$y+50}] 
	focus .modulselect	
	bind .modulselect <Key-Return> ModulSelect_OK

	#also borrowed from http://wiki.tcl.tk/534
	# This should display $child centered in $parent
	set w [winfo width $parent]
	set h [winfo height $parent]
	set x [winfo rootx $parent]
	set y [winfo rooty $parent]
	set xpos "+[ expr {$x+($w-[winfo width $child])/2}]"
	set ypos "+[ expr {$y+($h-[winfo height $child])/2}]"
	wm geometry $child "$xpos$ypos"

	tkwait window .modulselect
}

#OK callback of the ModulSelect Dlg
proc ModulSelect_OK {} {
	global AvailableModules 
	.menubar.modules.menu delete 1 end
	.menubar.modules.menu add command -label "Select Modules" -command { ShowModulSelect }
	.menubar.modules.menu add separator
	
	foreach modul $AvailableModules {
		global $modul\_selected
		if ($$modul\_selected==1) {
			$modul\_AddMenu
		}
	}
	destroy .modulselect
}

# make window


frame .mainframe -width 10c -height 5c
wm title . "AVRConfig pre2 0.1"
pack .mainframe

frame .edf 

text .ed -width 80 -height 20 -bg grey \
-yscrollcommand ".ys set" 

scrollbar .ys -command ".ed yview" 
pack .ed .ys -in .edf -side left -fill y

frame .menubar -relief raised -bd 2
pack .menubar -in .mainframe -fill x
pack .edf -in .mainframe -after .menubar -fill x


# make menu

menubutton .menubar.file -text File -underline 0 -menu .menubar.file.menu
menu .menubar.file.menu
.menubar.file.menu add command -label "Save" -command { Save }

menubutton .menubar.modules -text Modules -menu .menubar.modules.menu
pack .menubar.file .menubar.modules -side left
menu .menubar.modules.menu 
.menubar.modules.menu add command -label "Select Modules" -command { ShowModulSelect }
.menubar.modules.menu add separator


# include modules and execute init-functions
cd modules
set modules [glob *]
foreach module $modules {
	source $module
	set AvailableModules [linsert $AvailableModules end $module]
	$module\_init
	set $module\_selected 0
}
cd ..